/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const classroomApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const addMissing = require(`${libs}/middlewares/addMissing`);

const config = require(`${libs}/config`);

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

let rqfCount = new ReqQueryFields();

//classroomApp.use(cache('15 day'));


// Complete range of the enrollments dataset.
// Returns a tuple of start and ending years of the complete enrollments dataset.
classroomApp.get('/year_range', (req, res, next) => {
    req.sql.from('escola_agregada')
    .field('MIN(escola_agregada.ano_censo)', 'start_year')
    .field('MAX(escola_agregada.ano_censo)', 'end_year');
    next();
}, query, response('range'));

classroomApp.get('/years', (req, res, next) => {
    req.sql.from('escola_agregada')
    .field('DISTINCT escola_agregada.ano_censo', 'year');
    next();
}, query, response('years'));

classroomApp.get('/source', (req, res, next) => {
    req.sql.from('fonte')
    .field('fonte', 'source')
    .where('tabela = \'escola_agregada\'');
    next();
}, query, response('source'));

classroomApp.get('/adm_dependency', (req, res, next) => {
    req.result = [];
    for(let i = 1; i <= 4; ++i) {
        req.result.push({
            id: i,
            name: id2str.admDependency(i)
        });
    };
    next();
}, response('adm_dependency'));

classroomApp.get('/adm_dependency_detailed', (req, res, next) => {
    req.result = [];
    for(let i = 1; i <= 6; ++i) {
        req.result.push({
            id: i,
            name: id2str.admDependencyPriv(i)
        });
    };
    next();
}, response('adm_dependency_detailed'));

classroomApp.get('/location', (req, res, next) => {
    req.result = [
		{id: 1, name: 'Urbana'},
		{id: 2, name: 'Rural'}
	];
	next();
}, response('location'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValueToField({
    name: 'school',
    table: 'escola_agregada',
    tableField: ['nome_escola', 'id'],
    resultField: ['school_name', 'school_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    }
}, 'dims').addValueToField({
    name: 'school',
    table: 'escola_agregada',
    tableField: 'nome_escola',
    resultField: 'school_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    }
}, 'filter').addValueToField({
    name: 'city',
    table: 'municipio',
    tableField: ['nome', 'id'],
    resultField: ['city_name', 'city_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'escola_agregada'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'escola_agregada'
    }
}, 'dims').addValueToField({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'escola_agregada'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'escola_agregada'
    }
}, 'filter').addValue({
    name: 'state',
    table: 'estado',
    tableField: ['nome', 'id'],
    resultField: ['state_name', 'state_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: 'escola_agregada'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'escola_agregada'
    }
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: ['nome', 'id'],
    resultField: ['region_name', 'region_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'regiao_id',
        foreignTable: 'escola_agregada'
    }
}).addValue({
    name: 'min_year',
    table: 'escola_agregada',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'escola_agregada',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'adm_dependency',
    table: 'escola_agregada',
    tableField: 'dependencia_adm_id',
    resultField: 'adm_dependency_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'dependencia_adm_id'
    }
}).addValue({
    name: 'adm_dependency_detailed',
    table: 'escola_agregada',
    tableField: 'dependencia_adm_priv',
    resultField: 'adm_dependency_detailed_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'dependencia_adm_priv'
    }
}).addValue({
    name: 'location',
    table: 'escola_agregada',
    tableField: 'localizacao_id',
    resultField: 'location_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'localizacao_id'
    }
});

classroomApp.get('/', cache('15 day'), rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.from('escola_agregada')
        .field('SUM(escola_agregada.qtde_salas_utilizadas_dentro)', 'total')
        .field("'Brasil'", 'name')
        .field('escola_agregada.ano_censo', 'year')
        .group('escola_agregada.ano_censo')
        .order('escola_agregada.ano_censo')
        .where('escola_agregada.situacao_funcionamento_pareada = 1 AND escola_agregada.local_func_predio_escolar = 1')
        .where('escola_agregada.ensino_regular = 1 OR escola_agregada.ensino_eja = 1 OR escola_agregada.educacao_profissional = 1');
    next();
}, query, addMissing(rqf), id2str.transform(), (req, res, next) => {
    if (req.dims.location && req.result.length < 2) {    // Garantimos que conterá as duas localizações no resultado para o simCAQ
        let result = [];
        for (let i = 1; i <= 2; i++) {
            if (i !== req.result[0].location_id) {
                let newObj = Object.assign({}, req.result[0]);
                newObj.location_id = i;
                newObj.total = 0;
                newObj.location_name = (i == 1) ? "Urbana" : "Rural";
                result.push(newObj);
            }
            else {
                result.push(req.result[0]);
            }
        }

        req.result = result;
    }

    next();
}, response('classroom'));

module.exports = classroomApp;
